using System;
using System.Collections;
using System.Text;
using System.Xml;
using System.Net;
using System.Net.Sockets;
using Socket = System.Net.Sockets.Socket;
using System.IO;


using Microsoft.SPOT;
using Microsoft.SPOT.Presentation;
using Microsoft.SPOT.Hardware;

namespace RSSDeneme
{
    class FeedItem
    {
        public String Title = String.Empty;
        public String Link = String.Empty;
        public String Description = String.Empty;
        public String pubDate = String.Empty;
    }

    class RssReader
    {

        public static bool ParseFeed(String XML, out string FeedItems, out string FeedTitle)
        {
            // Quick and dirty solution for parsing ISO-8859-1 data
            if (XML.IndexOf("encoding=\"ISO-8859-1\"") != -1)
            {
                // Just replace ISO-8859-1 with utf-8
                XML = XML.Substring(0, XML.IndexOf("encoding=\"ISO-8859-1\"")) + "encoding=\"utf-8\"" + XML.Substring(XML.IndexOf("encoding=\"ISO-8859-1\"")+21);
            }

            // Create Memory stream
            System.Text.UTF8Encoding utf8 = new System.Text.UTF8Encoding();

            byte[] data = System.Text.Encoding.UTF8.GetBytes(XML);

            MemoryStream xmlStream = new MemoryStream(data);


            // Create Xml Reader
            XmlTextReader xmlDoc;

            // Create ArrayList to return
            FeedItems = String.Empty;

            // Create FeedTitle
            FeedTitle = String.Empty;

            try
            {
                xmlDoc = new XmlTextReader(xmlStream);
            }
            catch
            {
                // Invalid XML document, return false
                return false;
            }

            // Storage for current depth when reading Channel
            int ChannelDepth = 0;

            // Storage for current depth when reading Item
            int ItemDepth = 0;

            while (xmlDoc.Read())
            {
                if (xmlDoc.Name == "channel")
                {
                    // Store channel depth
                    ChannelDepth = xmlDoc.Depth;

                    // Advance to next node
                    xmlDoc.Read();

                    // Read all elements in Channel tag
                    while (xmlDoc.Depth != ChannelDepth)
                    {
                        // FeedTitle
                        if (xmlDoc.Name == "title")
                        {
                            // Make sure where reading the title tag of channel and not a subtag
                            if (xmlDoc.Depth == (ChannelDepth + 1))
                                FeedTitle = xmlDoc.ReadString();
                        }

                        if (xmlDoc.Name == "item")
                        {
                            // Store item depth
                            ItemDepth = xmlDoc.Depth;

                            // Advance to next node
                            xmlDoc.Read();

                            // Create item object
                            FeedItem Item = new FeedItem();

                            // Read all elements in Item tag
                            while (xmlDoc.Depth != ItemDepth)
                            {
                                // Title tag
                                if (xmlDoc.Name == "title")
                                {
                                    Item.Title = xmlDoc.ReadString();
                                }

                                // Description
                                if (xmlDoc.Name == "description")
                                {
                                    Item.Description = xmlDoc.ReadString();
                                }

                                // Publication date
                                if (xmlDoc.Name == "pubDate")
                                {
                                    Item.pubDate = xmlDoc.ReadString();
                                }

                                // Link
                                if (xmlDoc.Name == "link")
                                {
                                    Item.Link = xmlDoc.ReadString();
                                }

                                // Read next node
                                xmlDoc.Read();
                            }

                            // Add Item to ArrayList
                            FeedItems = Item.Description;
                        }


                        xmlDoc.Read();
                    }


                }

            }


            return true;

        }


        /// <summary>
        /// Download RSS Feed from server
        /// Supports Location: response header. 
        /// Only Port 80/http is supported
        /// </summary>
        /// <param name="Server">URL</param>
        /// <returns>RSS Feed as String</returns>
        public static String GetXML(String URL, String Referer)
        {
            // Print URL
            Debug.Print("RssReader::getXML: [" + URL + "]");

            // Check for HTTP and remove it from the URL
            if (URL.IndexOf("http://") != 0) { return ""; }
            URL = URL.Substring(7);

            // Get Server
            String Server = URL.Substring(0, URL.IndexOf("/"));

            // Get Path
            String Path = URL.Substring(URL.IndexOf("/"));
            
            // Get Port
            Int32 Port = 80;
            if (Server.IndexOf(':') != -1)
            {
                String[] sServer = Server.Split(new char[] {'|'});
                Server = sServer[0];
                Port = Str_Int_Func.StrVal(sServer[1]);
            }
            
            // Server settings
            const Int32 BufferSize = 128;

            // String used to hold the response
            String Response = String.Empty;

            // Create Socket, used in a using block so Socket is automaticly closed
            using (Socket requestSocket = ConnectSocket(Server, Port))
            {
                // Create request header
                String request = "GET " + Path + " HTTP/1.1\r\n";                                       // Request URI
                request += "Host: " + Server + "\r\n";                                                  // HTTP1.1 Host
                request += "Connection: Close\r\n";                                                     // HTTP1.1 Close connection after request
                request += "Accept-Charset: utf-8;\r\n";                                     // HTTP1.1 Accept-Charset
                request += "Accept: application/rss+xml, application/xhtml+xml, text/html\r\n";         // HTTP1.1 Accept
                request += "Pragma:	no-cache\r\n";
                request += "Cache-Control: no-cache\r\n";
                request += "User-Agent: Michael Schwarz (.NET Micro Framework 3.0) from celiker.com\r\n";

                if (Referer != String.Empty)
                {
                    request += "Referer: " + Referer + "\r\n";                                          // HTTP1.1 Referer
                }

                request += "\r\n";                                                                      // End of header


                // Send request header
                Byte[] requestData = Encoding.UTF8.GetBytes(request);
                requestSocket.Send(requestData, requestData.Length, 0);

                // Buffer used for recieving 
                Byte[] recBuffer = new Byte[BufferSize];

                // Wait 30 seconds for data 
                DateTime timeoutAt = DateTime.Now.AddSeconds(30);
                while (requestSocket.Available == 0 && DateTime.Now < timeoutAt)
                {
                    System.Threading.Thread.Sleep(100);
                }

                // Recieve Data
                while (requestSocket.Poll(2 * 1000000, SelectMode.SelectRead))
                {
                    // Clear recieve buffer
                    Array.Clear(recBuffer, 0, recBuffer.Length);

                    // Read data into buffer
                    Int32 bytesRead = requestSocket.Receive(recBuffer);

                    // Check if we have bytes in the buffer, if not, stop reading
                    if (bytesRead == 0)
                        break;

                    // Add recieved data to the response string
                    try
                    {
                        Response = Response + new String(Encoding.UTF8.GetChars(recBuffer));
                    } catch {}

                }


                // Get response header
                String ResponseHeader = String.Empty;
                if (Response.IndexOf("\r\n\r\n") != -1) { ResponseHeader = Response.Substring(0, Response.IndexOf("\r\n\r\n")); }

                // Check for a location header
                if (ResponseHeader.IndexOf("Location: ") != -1)
                {
                    // Get Location
                    Int32 LocationStart = ResponseHeader.IndexOf("Location: ") + 10;
                    Int32 LocationEnd = ResponseHeader.IndexOf("\r\n", LocationStart);

                    String newLocation = ResponseHeader.Substring(LocationStart, LocationEnd - LocationStart);

                    // Retrieve and return from the new location
                    return GetXML(newLocation,URL);
                }

                // Strip header from response
                if (Response.IndexOf("\r\n\r\n") != -1)
                {
                    Response = Response.Substring(Response.IndexOf("\r\n\r\n") + 4);
                }

                // Return Response
                return Response;
            }
        }


        /// <summary>
        /// Open Socket to given Server and Port
        /// From: SocketClient example
        /// </summary>
        /// <param name="server">Server</param>
        /// <param name="port">Port</param>
        /// <returns>New Socket</returns>
        private static Socket ConnectSocket(String server, Int32 port)
        {
            // Get server's IP address.
            IPHostEntry hostEntry = Dns.GetHostEntry(server);

            // Create socket and connect to the server's IP address and port
            Socket socket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            socket.Connect(new IPEndPoint(hostEntry.AddressList[0], port));
            return socket;
        }
    }
}
